-- written by groverbuger for g3d
-- may 2021
-- MIT license

local g3d = require "g3d"
local earth = g3d.newModel("assets/box.obj", "assets/level1_walls.png", {4,0,0}, {math.pi/2,0,0}, 0.9)
local moon = g3d.newModel("assets/sphere.obj", "assets/moon.png", {0,0,0}, nil, {0.5,0.5,0.5})
local plane = g3d.newModel("assets/plane.obj", "assets/floor.png", {0,0,-4},{0,-math.pi,0}, 100)
local timer = 0
local color = {1,0.7,0.3}
local ambientcolor = {1,1,1}

local lightingShader = love.graphics.newShader(g3d.path .. "/g3d.vert", "lighting.frag")
local depthShader = love.graphics.newShader(g3d.path .. "/g3d.vert","/sfbo.frag")

local canvas = love.graphics.newCanvas(1024,1024)
local SFBO = love.graphics.newCanvas(1024,1024, {readable=true, format = "depth24"})


--earth:makeNormals(true)

function love.update(dt)
    timer = timer + dt
    moon:setTranslation(math.cos(timer)*2+4 , math.sin(timer)*2, 0)
    moon:setRotation(0, timer - math.pi/2, 0)
    g3d.camera.firstPersonMovement(dt)
    if love.keyboard.isDown "escape" then
        love.event.push "quit"
    end
	lightingShader:send("lightPosition", g3d.camera.position)
	--lightingShader:send("lightPosition", moon.translation)
	lightingShader:send("lightColor", color)
	lightingShader:send("ambientLightColor", ambientcolor)
	lightingShader:send("ambient", 0.2)
	lightingShader:send("viewPos", g3d.camera.position)
	
	depthShader:send("depthMap", SFBO)
end

function love.draw()
    love.graphics.setCanvas({canvas, depth=true, depthstencil=SFBO})
	    love.graphics.clear();
		g3d.camera.updateViewMatrix(depthShader)
		g3d.camera.updateProjectionMatrix(depthShader)
		earth:draw(depthShader)
		moon:draw(depthShader)
		plane:draw(depthShader)
    love.graphics.setCanvas()

	
    g3d.camera.updateViewMatrix(lightingShader)
    g3d.camera.updateProjectionMatrix(lightingShader)
    earth:draw(lightingShader)
    moon:draw(lightingShader)
    plane:draw(lightingShader)	
	love.graphics.draw(canvas,0,0,0,0.25)
end

function love.mousemoved(x,y, dx,dy)
    g3d.camera.firstPersonLook(dx,dy)
end

